@@ -10,11 +10,14 @@ The pyoperator package contains the following modules or packages:
 - operators_pywt : (optional) loaded if PyWavelets is present.
 """
 
+from .config import VERSION as __version__
 from .utils import *
 from .core import *
+from .fft import *
 from .linear import *
 from .nonlinear import *
-from .iterative import *
+from . import iterative
+from .iterative import pcg
 
 try:
     from .operators_pywt import *
@@ -26,9 +29,9 @@ try:
 except(ImportError):
     pass
 
-__version__ = '0.5'
-
 import types
 __all__ = [ f for f in dir() if f[0] != '_' and not isinstance(eval(f),
             types.ModuleType)]
+
+del f #XXX not necessary with Python3
 del types
