@@ -10,7 +10,7 @@ from .core import (Operator, CompositionOperator, DiagonalOperator,
 from .decorators import (aligned, contiguous, inplace, linear, real, square,
                          unitary)
 from .memory import empty
-from .utils import openmp_num_threads, product, tointtuple
+from .utils import isalias, openmp_num_threads, product, tointtuple
 from .utils.ufuncs import multiply_conjugate
 
 __all__ = ['ConvolutionOperator', 'FFTOperator']
@@ -41,8 +41,9 @@ class _FFTWConvolutionOperator(Operator):
     using the discrete Fourier transform.
         
     """
-    def __init__(self, kernel, shapein=None, dtype=None, axes=None,
-                 fftw_flag='FFTW_MEASURE', nthreads=None, **keywords):
+    def __init__(self, kernel, shapein=None, axes=None,
+                 fftw_flag='FFTW_MEASURE', nthreads=None, dtype=None,
+                 **keywords):
         """
         Parameters
         ----------
@@ -50,8 +51,6 @@ class _FFTWConvolutionOperator(Operator):
             The multi-dimensional convolution kernel.
         shapein : tuple
             The shape of the input to be convolved by the kernel.
-        dtype : dtype
-            Operator's dtype.
         axes : tuple
             Axes along which the convolution is performed. Convolution over
             less axes than the operator's input is not yet supported.
@@ -65,6 +64,8 @@ class _FFTWConvolutionOperator(Operator):
         nthreads : int
             Tells how many threads to use when invoking FFTW or MKL. Default is
             the number of cores.
+        dtype : dtype
+            Operator's dtype.
 
         """
         kernel = np.array(kernel, dtype=dtype, copy=False)
@@ -114,8 +115,7 @@ class _FFTWComplexConvolutionOperator(_FFTWConvolutionOperator):
 
     """
     def __init__(self, kernel, shapein, axes, fftw_flag, nthreads, **keywords):
-        dtype = kernel.dtype
-        _FFTWConvolutionOperator.__init__(self, kernel, shapein, dtype, axes,
+        _FFTWConvolutionOperator.__init__(self, kernel, shapein, axes,
                                           fftw_flag, nthreads, **keywords)
         n = product(shapein)
         fft = _FFTWComplexForwardOperator(self.shapein, self.axes,
@@ -134,8 +134,7 @@ class _FFTWRealConvolutionOperator(_FFTWConvolutionOperator):
     
     """
     def __init__(self, kernel, shapein, axes, fftw_flag, nthreads, **keywords):
-        dtype = kernel.dtype
-        _FFTWConvolutionOperator.__init__(self, kernel, shapein, dtype, axes,
+        _FFTWConvolutionOperator.__init__(self, kernel, shapein, axes,
                                           fftw_flag, nthreads, **keywords)
         self.set_rule('.{_FFTWRealConvolutionOperator}', self.
                       _rule_real_convolution, CompositionOperator, globals())
@@ -144,10 +143,10 @@ class _FFTWRealConvolutionOperator(_FFTWConvolutionOperator):
         self.set_rule('{_FFTWComplexForwardOperator}.', self.
                       _rule_complex_forward, CompositionOperator, globals())
 
-        dtype_ = np.dtype('complex' + str(int(dtype.name[5:]) * 2))
-        shape_ = self._reshape_to_halfstorage(shapein)
+        dtype_ = np.dtype('complex' + str(int(self.dtype.name[5:]) * 2))
+        shape_ = self._reshape_to_halfstorage(self.shapein)
         _load_wisdom()
-        with _pool.get(shapein, dtype) as in_:
+        with _pool.get(self.shapein, self.dtype) as in_:
             with _pool.get(shape_, dtype_) as out:
                 t0 = time.time()
                 self._fplan = pyfftw.FFTW(in_, out, axes=self.axes,
@@ -162,8 +161,8 @@ class _FFTWRealConvolutionOperator(_FFTWConvolutionOperator):
         if time.time() - t0 > FFTW_WISDOM_MIN_DELAY:
             _save_wisdom()
 
-        kernel_fft = _get_kernel_fft(kernel, shapein, dtype, shape_, dtype_,
-                                     self._fplan)
+        kernel_fft = _get_kernel_fft(kernel, self.shapein, self.dtype, shape_,
+                                     dtype_, self._fplan)
         kernel_fft /= product(shapein)
         self.kernel = kernel_fft
 
@@ -233,7 +232,7 @@ class _FFTWComplexOperator(Operator):
         Operator.__init__(self, shapein=shapein, dtype=dtype, **keywords)
 
     def direct(self, input, output):
-        if self.isalias(input, output):
+        if isalias(input, output):
             self.iplan.update_arrays(input, output)
             self.iplan.execute()
         else:
