@@ -8,14 +8,17 @@ from scipy.sparse.linalg import eigsh
 from .decorators import inplace, linear, real, square, symmetric
 from .core import (Operator, BlockRowOperator, BroadcastingOperator,
                    CompositionOperator, DiagonalOperator, ReductionOperator,
-                   DirectOperatorFactory, asoperator)
-from .utils import cast
+                   Variable, DirectOperatorFactory, ReverseOperatorFactory,
+                   asoperator, X)
+from .utils import cast, ifind, tointtuple
 
 __all__ = [
     'BandOperator',
     'DiagonalNumexprOperator',
     'DiagonalNumexprNonSeparableOperator',
+    'DifferenceOperator',
     'EigendecompositionOperator',
+    'EinsteinSumOperator',
     'IntegrationTrapezeWeightOperator',
     'PackOperator',
     'SumOperator',
@@ -140,6 +143,124 @@ class DiagonalNumexprNonSeparableOperator(DiagonalOperator):
         return None
 
 
+@linear
+class EinsteinSumOperator(Operator):
+    """
+    Einstein sum operator.
+
+    Example
+    -------
+    >>> op = EinsteinSumOperator('ii->i', X, shapein=(4,4))
+    >>> op(np.arange(16).reshape(4,4))
+    [0., 5., 10., 15.]
+
+    """
+    def __init__(self, subscripts, *operands, **keywords):
+        subscripts = subscripts.replace(' ', '')
+        if '->' not in subscripts:
+            raise NotImplementedError('Unlike einsum, the output subscript cann'
+                                      'ot yet be specified.')
+        in_, out = subscripts.split('->')
+        ins = in_.split(',')
+        if len(operands) == len(ins) - 1 and X not in operands:
+            operands = operands + (X,)
+        if len(ins) != len(operands):
+            raise ValueError("The number of input operands '{0}' does not match"
+                             " that from the subscripts '{1}'.".format(len(
+                             operands), len(ins)))
+        self.iX = ifind(operands, lambda x: x is X)
+        self.operands = operands
+        self.subscripts = subscripts
+        self.lsubscripts = tuple([self._get_subscript_as_list(s)
+                                  for s in ins + [out]])
+        Operator.__init__(self, **keywords)
+        tsubscripts = self._get_reversed_subscripts()
+        if self.subscripts == '...jk,...k->...j':
+            tsubscripts = '...kj,...k->...j'
+        elif self.subscripts == '...kj,...k->...j':
+            tsubscripts = '...jk,...k->...j'
+        else:
+            raise NotImplementedError()
+        self.set_rule('.T', lambda s: ReverseOperatorFactory(
+            EinsteinSumOperator, s, tsubscripts, *self.operands))
+
+    def direct(self, input, output):
+        operands = list(self.operands)
+        operands[self.iX] = input
+        np.einsum(self.subscripts, *operands, out=output)
+
+    def reshapein(self, shapein):
+        shapes = [None if isinstance(o, Variable) else o.shape
+                  for o in self.operands] + [None]
+        shapes[self.iX] = shapein
+        return self._get_shape(shapes, -1)
+
+    def reshapeout(self, shapeout):
+        shapes = [None if isinstance(o, Variable) else o.shape
+                  for o in self.operands] + [shapeout]
+        return self._get_shape(shapes, self.iX)
+
+    @staticmethod
+    def _enumerate_ellipsis(l):
+        """
+        Enumerate a sequence that may include an Ellipsis, in which case
+        a slice is returned and negative indices afterwards.
+
+        Example
+        -------
+        >>> list(loop_ellipsis(['a','b',Ellipsis,'c']))
+        [0, 1, slice(2, -1, None), -1]
+
+        """
+        i = 0
+        for x in l:
+            if x is not Ellipsis:
+                yield i, l[i]
+                i += 1
+            else:
+                yield slice(i, i - len(l) + 1), Ellipsis
+                i -= len(l) - 1
+
+    def _get_reversed_subscripts(self):
+        ls = self.lsubscripts
+        ls_reversed = [l[::-1] for l in ls[-2::-1]] + [ls[-1][::-1]]
+        ls_reversed = [''.join(['...' if a is Ellipsis else a for a in l])
+                       for l in ls_reversed]
+        return ','.join(ls_reversed[:-1]) + '->' + ls_reversed[-1]
+
+    def _get_shape(self, shapes, i):
+        """ Infer ith shape from the shapes of the other operands. """
+        shape = []
+        #print 'shapes', shapes
+        for iaxis, subscript in self._enumerate_ellipsis(self.lsubscripts[i]):
+            # find this subscript elsewhere
+            #print 'iaxis, subscript',  iaxis, subscript
+            try:
+                for iother, subscripts_other in enumerate(self.lsubscripts):
+                    #print 'iother, sub_other', iother, subscripts_other, 'skipped' if iother == i or shapes[iother] is None else 'looking...'
+                    if iother == i or shapes[iother] is None:
+                        continue
+                    for iaxis_other, subscript_other in \
+                        self._enumerate_ellipsis(subscripts_other):
+                        #print 'iaxis_other, subscript_other:', iaxis_other, subscript_other
+                        if subscript == subscript_other:
+                            new_dims = tointtuple(shapes[iother][iaxis_other])
+                            shape.extend(new_dims)
+                            raise StopIteration()
+                if i == -1:
+                    raise ValueError("Einstein sum subscripts include an output"
+                                     " subscript '{0}' which never appeared in "
+                                     "an input.".format(subscript))
+                return None
+            except StopIteration:
+                pass
+        return tuple(shape)
+        
+    def _get_subscript_as_list(self, s):
+        s = s.replace('...', '.')
+        return [l if l != '.' else Ellipsis for l in s]
+        
+
 class IntegrationTrapezeWeightOperator(BlockRowOperator):
     """
     Return weights as a block row operator to perform trapeze integration.
@@ -596,6 +717,41 @@ class SymmetricBandOperator(Operator):
             out = UpperTriangularOperator(self.shape, ab_chol, **self.kwargs)
         return out
 
+
+@real
+@linear
+class DifferenceOperator(Operator):
+    """
+    Non-optimised difference operator.
+
+    """
+    def __init__(self, axis=-1, **keywords):
+        self.axis = axis
+        Operator.__init__(self, **keywords)
+
+    def direct(self, input, output):
+        output[...] = np.diff(input, axis=self.axis)
+
+    def transpose(self, input, output):
+        slices = [slice(None)] * input.ndim
+        slices[self.axis] = slice(1, -1)
+        shapetmp = list(input.shape)
+        shapetmp[self.axis] += 2
+        tmp = np.zeros(shapetmp)
+        tmp[slices] = input
+        output[...] = -np.diff(tmp, axis=self.axis)
+
+    def reshapein(self, shapein):
+        shape = list(shapein)
+        shape[self.axis] -= 1
+        return tuple(shape)
+
+    def reshapeout(self, shapeout):
+        shape = list(shapeout)
+        shape[self.axis] += 1
+        return tuple(shape)
+
+
 @symmetric
 class EigendecompositionOperator(Operator):
     """
